"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var commander = require("commander");
var fs_extra_1 = require("fs-extra");
var path_1 = require("path");
var library_controller_1 = require("./core/library-controller");
var log_1 = require("./core/log");
var App = /** @class */ (function () {
    function App() {
        var _this = this;
        this.linkSourceAlias = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            //console.log(args[0], args[1])
            _this.link(args[0], { source: true, all: args[0] == undefined });
        };
        this.linkDistAlias = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            _this.link(args[0], { dest: true, all: args[0] == undefined });
        };
        this.link = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var n = args[0];
            var d = args[1].dest === true;
            var s = args[1].source === true;
            var a = args[1].all === true;
            if (d && s) {
                _this.exitError("Only one option must be set (--dest or --source)");
            }
            if (n !== undefined && a) {
                _this.exitError("Only one option must be set (library or --all)");
            }
            var lg = new library_controller_1.LibraryController();
            var mode = s ? "source" : "dest";
            lg.check().subscribe(function (success) {
                if (a)
                    lg.linkAll(mode);
                else
                    lg.link(n, mode);
                log_1.log("Libraries linked to " + mode, log_1.ThemeColors.info);
            }, _this.exitError);
        };
        this.build = function (name, commands) {
            var lg = new library_controller_1.LibraryController();
            lg.check().subscribe(function (success) {
                if (lg.hasLibrary(name)) {
                    lg.buildLibrary(name)
                        .subscribe(_this.exit, _this.exitError);
                }
                else {
                    lg.buildAll().subscribe(function (lib) {
                        log_1.log(lib.root + " build complete");
                    }, _this.exitError);
                }
            }, _this.exitError);
        };
        this.deleteLib = function (name) {
            var lg = new library_controller_1.LibraryController();
            lg.check().subscribe(function (success) {
                lg.deleteLib(name).subscribe(function (success) {
                    log_1.log("Library deleted", log_1.ThemeColors.info);
                    _this.exit();
                }, _this.exitError);
            }, _this.exitError);
        };
        this.exitError = function (err) {
            log_1.log(err, log_1.ThemeColors.error);
            _this.exit(false);
        };
        this.subPackage = function (libName, subName) {
            var lg = new library_controller_1.LibraryController();
            lg.check().subscribe(function (success) {
                lg.createScoped(libName, subName);
            }, _this.exitError);
        };
        this.createLib = function (name) {
            var lg = new library_controller_1.LibraryController();
            lg.check().subscribe(function (success) {
                lg.create(name)
                    .subscribe(function (success) {
                    log_1.log("Library configuration updated", log_1.ThemeColors.info);
                    _this.exit();
                }, _this.exitError);
            }, _this.exitError);
        };
        this.addDependency = function (target, value) {
            var lg = new library_controller_1.LibraryController();
            lg.check().subscribe(function (success) {
                lg.addPeerDependency(target, value)
                    .subscribe(function (success) {
                    log_1.log("Peer dependecy added", log_1.ThemeColors.info);
                    _this.exit();
                }, _this.exitError);
            }, _this.exitError);
        };
        this.exit = function (success) {
            if (success === void 0) { success = true; }
            process.exit(success ? 0 : 1);
        };
    }
    App.prototype.initialize = function () {
        var pkg = fs_extra_1.readJsonSync(path_1.resolve(__dirname, "..", "package.json"));
        commander
            .version(pkg.version);
        commander.command("link [library]")
            .description("Link one or all libraries.")
            .action(this.link)
            .option('-s --source', 'link to source')
            .option('-d --dest', 'link to dist')
            .option('-a --all', 'on all library projects defined in the angular.json file.');
        commander.command("ls [library]")
            .description("Link to source for one or all libraries.")
            .action(this.linkSourceAlias)
            .option('-a --all', 'on all library projects defined in the angular.json file.');
        commander.command("ld [library]")
            .description("Link to dist for one or all libraries.")
            .action(this.linkDistAlias)
            .option('-s --source', 'link to source')
            .option('-d --dest', 'link to dist')
            .option('-a --all', 'on all library projects defined in the angular.json file.');
        commander.command('sub <library> <name>')
            .action(this.subPackage);
        commander.command("new [library]")
            .description("Create an angular library.")
            .action(this.createLib);
        commander.command("rm [library]")
            .description("Delete an angular library.")
            .action(this.deleteLib);
        commander.command("dep [target] [value]")
            .description("Add value to target peer dependencies.")
            .action(this.addDependency);
        commander.command("build [library]")
            .description("Build one or all libraries.")
            .action(this.build);
        commander.parse(process.argv);
    };
    return App;
}());
exports.App = App;
//# sourceMappingURL=app.js.map