"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Package = /** @class */ (function () {
    function Package(name) {
        this.name = name;
        this.dependencies = [];
    }
    return Package;
}());
var DependenciesResolver = /** @class */ (function () {
    function DependenciesResolver() {
        this.services = {};
    }
    DependenciesResolver.prototype.add = function (name) {
        this.addAndGet(name);
    };
    DependenciesResolver.prototype.addAndGet = function (serviceName) {
        if (this.services[serviceName]) {
            return this.services[serviceName];
        }
        this.services[serviceName] = new Package(serviceName);
        //Add dependency to root element for sort function to work
        if (serviceName !== DependenciesResolver.ROOT_SERVICE_NAME) { //avoid circular depdency of root service
            this.setDependency(DependenciesResolver.ROOT_SERVICE_NAME, serviceName);
        }
        return this.services[serviceName];
    };
    DependenciesResolver.prototype.setDependency = function (serviceName, dependencyName) {
        var service = this.addAndGet(serviceName);
        var dependency = this.addAndGet(dependencyName);
        service.dependencies.push(dependency);
    };
    DependenciesResolver.prototype.resolve = function (serviceName) {
        var resolved = [];
        var unresolved = [];
        var service = this.services[serviceName];
        if (!service) {
            throw new Error('DepService ' + serviceName + ' does not exist');
        }
        this.recursiveResolve(service, resolved, unresolved);
        return resolved.map(function (s) { return s.name; });
    };
    DependenciesResolver.prototype.sort = function () {
        var deps = this.resolve(DependenciesResolver.ROOT_SERVICE_NAME);
        deps.pop(); //remove DependencyResolver.ROOT_SERVICE_NAME element
        return deps;
    };
    DependenciesResolver.prototype.recursiveResolve = function (service, resolved, unresolved) {
        var _this = this;
        unresolved.push(service);
        service.dependencies.forEach(function (sub) {
            if (resolved.indexOf(sub) === -1) {
                if (unresolved.indexOf(sub) !== -1) {
                    throw new Error('Circular reference detected: ' + service.name + ' -> ' + sub.name);
                }
                _this.recursiveResolve(sub, resolved, unresolved);
            }
        });
        resolved.push(service);
        unresolved.splice(unresolved.indexOf(service), 1);
    };
    DependenciesResolver.ROOT_SERVICE_NAME = '#root#';
    return DependenciesResolver;
}());
exports.DependenciesResolver = DependenciesResolver;
//# sourceMappingURL=dependency-resolver.js.map