"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var fs = require("fs-extra");
var path = require("path");
var rimraf = require("rimraf");
var jsonStringify = require("json-stable-stringify");
var find = require("find");
var exec_1 = require("./exec");
var dependency_resolver_1 = require("./dependency-resolver");
var log_1 = require("./log");
var PATH_SEPARATOR = "/";
var PATH_SEPARATOR_RE = /\\/g;
// mystring.replace(/\/\//g, "/");
var pathJoin = function () {
    var values = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        values[_i] = arguments[_i];
    }
    return values.join(PATH_SEPARATOR);
};
var pathNormalize = function (value) {
    return path.normalize(value).replace(PATH_SEPARATOR_RE, PATH_SEPARATOR);
};
var findInParentRecurse = function (filename, dir) {
    var fn = null;
    var exist = function (fn) {
        return fs.existsSync(fn);
    };
    var check = function () {
        if (!exist(dir)) {
            fn = null;
            return;
        }
        fn = path.resolve(dir, filename);
        if (exist(fn)) {
            return;
        }
        var prev = dir;
        dir = path.resolve(dir, '..');
        if (prev == dir) {
            fn = null;
            return;
        }
        check();
    };
    check();
    return fn;
};
var JSON_CONF = {
    spaces: 2
};
var keySort = function (a, b) {
    return a.key < b.key ? -1 : 1;
};
var saveJson = function (filename, value) {
    fs.writeJSONSync(filename, JSON.parse(jsonStringify(value, keySort)), JSON_CONF);
};
var NG_JSON = "angular.json";
var TS_JSON = "tsconfig.json";
var PUBLIC_API_TS = "public_api.ts";
var PACKAGE_JSON = "package.json";
var TYPE_LIBRARY = "library";
var LibraryController = /** @class */ (function () {
    function LibraryController() {
        this.checked = false;
    }
    LibraryController.prototype.check = function () {
        var _this = this;
        this.cwd = process.cwd();
        return rxjs_1.Observable.create(function (o) {
            _this.ngPath = findInParentRecurse('angular.json', _this.cwd);
            if (_this.ngPath) {
                _this.ngAppDir = path.dirname(_this.ngPath);
                _this.ngConfig = fs.readJSONSync(NG_JSON);
                _this.tsConfig = fs.readJsonSync(TS_JSON);
                _this.checked = true;
                o.next(true);
                o.complete();
            }
            else {
                o.error("Angular project not found");
            }
        });
    };
    LibraryController.prototype.hasLibrary = function (name) {
        return (this.checked && this.libraryExists(name));
    };
    LibraryController.prototype.libraryExists = function (libName) {
        var lib = this.ngConfig.projects[libName];
        return (lib !== undefined && lib.projectType == TYPE_LIBRARY);
    };
    LibraryController.prototype.deleteLib = function (libName) {
        var _this = this;
        return rxjs_1.Observable.create(function (o) {
            var canEdit = _this.canEditLib(libName);
            if (typeof canEdit == "string")
                return o.error(canEdit);
            _this.cdProject();
            var ng = _this.ngConfig;
            var lib = ng.projects[libName];
            var libRoot = lib.root;
            delete (ng.projects[libName]);
            // update angular.json
            saveJson(NG_JSON, ng);
            var tj = _this.tsConfig;
            // delete tsconfig library path
            delete (tj.compilerOptions.paths[libName]);
            delete (tj.compilerOptions.paths[libName + "/*"]);
            // update tsconfig.json
            saveJson(TS_JSON, tj);
            // delete library directory
            rimraf(libRoot, function (error) {
                _this.restoreCwd();
                if (error)
                    return o.error(error);
                o.next(true);
                o.complete();
            });
        });
    };
    LibraryController.prototype.cdProject = function () {
        this.cwd = process.cwd();
        process.chdir(this.ngAppDir);
    };
    LibraryController.prototype.restoreCwd = function () {
        process.chdir(this.cwd);
    };
    LibraryController.prototype.createScoped = function (libName, scope) {
        var check = this.canEditLib(libName);
        if (check !== true)
            throw new Error(check);
        this.cdProject();
        var lib = this.ngConfig.projects[libName];
        var root = lib.root;
        var scopeDir = path.join(root, scope);
        fs.mkdirpSync(path.join(scopeDir, 'src'));
        process.chdir(scopeDir);
        var main = "src/" + scope + ".ts";
        var apiPath = 'src/' + PUBLIC_API_TS;
        fs.writeFileSync(apiPath, "export * from './" + scope + "'");
        fs.writeFileSync(main, "export const " + scope.toUpperCase() + " = \"" + scope + "\"");
        var pkg = {
            ngPackage: {
                lib: {
                    entryFile: apiPath
                }
            }
        };
        saveJson(PACKAGE_JSON, pkg);
        this.cdProject();
        var tsc = this.tsConfig;
        tsc.compilerOptions.paths[libName + "/" + scope] = [pathJoin(root, scope, apiPath)];
        saveJson(TS_JSON, tsc);
    };
    LibraryController.prototype.create = function (libName) {
        var _this = this;
        return rxjs_1.Observable.create(function (o) {
            if (!_this.checked)
                return o.error("Missing angular context");
            _this.cdProject();
            exec_1.exec("ng g library " + libName).subscribe(function (output) {
                console.log(output);
                // refresh angular.json
                _this.ngConfig = fs.readJSONSync(NG_JSON);
                // link tsconfig path to library entry file
                _this._linkSource(libName);
                _this.restoreCwd();
                o.next(true);
                o.complete();
            }),
                function (err) {
                    o.error(err);
                };
        });
    };
    LibraryController.prototype.getNgPackageJSON = function (libName) {
        var lib = this.ngConfig.projects[libName];
        return fs.readJSONSync(lib.architect.build.options.project);
    };
    LibraryController.prototype.getLibPackageJSON = function (libName) {
        var lib = this.ngConfig.projects[libName];
        return fs.readJSONSync(path.join(lib.root, PACKAGE_JSON));
    };
    LibraryController.prototype.saveLibPackageJSON = function (libName, pkg) {
        var lib = this.ngConfig.projects[libName];
        saveJson(path.join(lib.root, PACKAGE_JSON), pkg);
    };
    LibraryController.prototype.canEditLib = function (libName) {
        if (!this.checked)
            return "Missing angular context";
        if (!this.libraryExists(libName))
            return "Missing library";
        return true;
    };
    LibraryController.prototype._linkDist = function (libName, save) {
        if (save === void 0) { save = true; }
        var libPKG = this.getNgPackageJSON(libName);
        var lib = this.ngConfig.projects[libName];
        var dest = pathNormalize(pathJoin(lib.root, libPKG.dest));
        var tsj = this.tsConfig;
        process.chdir(lib.root);
        var pkgs = find.fileSync(PACKAGE_JSON, '.');
        pkgs = pkgs.filter(function (fn) {
            return fn != PACKAGE_JSON;
        });
        if (pkgs.length) {
            for (var _i = 0, pkgs_1 = pkgs; _i < pkgs_1.length; _i++) {
                var fn = pkgs_1[_i];
                fn = path.dirname(fn);
                delete tsj.compilerOptions.paths[libName + "/" + fn];
            }
        }
        this.cdProject();
        tsj.compilerOptions.paths[libName] = [dest];
        tsj.compilerOptions.paths[libName + "/*"] = [dest + "/*"];
        if (save)
            saveJson(TS_JSON, tsj);
    };
    LibraryController.prototype._linkSource = function (libName, save) {
        if (save === void 0) { save = true; }
        var lib = this.ngConfig.projects[libName];
        var entryFile = this.getEntryFile(libName);
        var tsj = this.tsConfig;
        tsj.compilerOptions.paths[libName] = [entryFile];
        delete (tsj.compilerOptions.paths[libName + "/*"]);
        process.chdir(lib.root);
        var pkgs = find.fileSync(PACKAGE_JSON, '.');
        pkgs = pkgs.filter(function (fn) {
            return fn != PACKAGE_JSON;
        });
        if (pkgs.length) {
            for (var _i = 0, pkgs_2 = pkgs; _i < pkgs_2.length; _i++) {
                var fn = pkgs_2[_i];
                var pkg = fs.readJSONSync(fn);
                var name_1 = path.dirname(fn);
                tsj.compilerOptions.paths[libName + "/" + name_1] = [pathJoin(lib.root, name_1, pkg.ngPackage.lib.entryFile)];
            }
        }
        this.cdProject();
        if (save)
            saveJson(TS_JSON, tsj);
    };
    LibraryController.prototype.getScopedEntryFile = function (libName) {
        var j = this.ngConfig;
        var lib = j.projects[libName];
        var libPKG = this.getNgPackageJSON(libName);
        var entryFile = lib.root + "/" + libPKG.lib.entryFile;
        var ex = path.extname(entryFile);
        return entryFile.slice(0, entryFile.lastIndexOf(ex));
    };
    /**
     *
     * @param name
     * @param type
     * @throws not checked
     */
    LibraryController.prototype.link = function (name, type) {
        if (!this.checked)
            throw new Error("Missing angular context");
        this.cdProject();
        if (type == "dest")
            this._linkDist(name);
        else
            this._linkSource(name);
        this.restoreCwd();
    };
    /**
     *
     * @param type
     * @throws not checked
     */
    LibraryController.prototype.linkAll = function (type) {
        if (!this.checked)
            throw new Error("Missing angular context");
        var libs = this.getLibraries();
        var fn = type == "source" ? this._linkSource : this._linkDist;
        for (var n in libs) {
            fn.apply(this, [n, false]);
        }
        saveJson(TS_JSON, this.tsConfig);
    };
    LibraryController.prototype.addPeerDependency = function (target, value) {
        var _this = this;
        return rxjs_1.Observable.create(function (o) {
            if (!_this.libraryExists(target) || !_this.libraryExists(value))
                return o.error("unknow library");
            _this.cdProject();
            var targetPKG = _this.getLibPackageJSON(target);
            var depPKG = _this.getLibPackageJSON(value);
            targetPKG.peerDependencies[value] = "^" + depPKG.version;
            _this.saveLibPackageJSON(target, targetPKG);
            _this.restoreCwd();
            o.next(true);
            o.complete();
        });
    };
    LibraryController.prototype.getLibraries = function () {
        var res = {};
        var libs = this.ngConfig.projects;
        for (var n in libs) {
            if (libs[n].projectType == TYPE_LIBRARY)
                res[n] = libs[n];
        }
        return res;
    };
    LibraryController.prototype.buildAll = function () {
        var _this = this;
        return rxjs_1.Observable.create(function (o) {
            var res = new dependency_resolver_1.DependenciesResolver();
            var libs = _this.getLibraries();
            var libNames;
            for (var n in libs) {
                res.add(n);
            }
            for (var n in libs) {
                var pkg = _this.getLibPackageJSON(n);
                for (var d in pkg.peerDependencies) {
                    if (libs[d] !== undefined)
                        res.setDependency(n, d);
                }
            }
            try {
                libNames = res.sort();
            }
            catch (error) {
                return o.error(error);
            }
            var next = function () {
                if (!libNames.length) {
                    return o.complete();
                }
                var name = libNames.shift();
                _this.buildLibrary(name)
                    .subscribe(function (success) {
                    o.next(libs[name]);
                    next();
                }, o.error);
            };
            next();
        });
    };
    LibraryController.prototype.buildLibrary = function (name) {
        return exec_1.exec("ng build " + name, true, false)
            .pipe(operators_1.map(function (output) {
            log_1.log(log_1.logMessage("✓", log_1.ThemeColors.info) + " Built " + name);
            return true;
        }));
    };
    LibraryController.prototype.getEntryFile = function (libName) {
        var j = this.ngConfig;
        var lib = j.projects[libName];
        var libPKG = this.getNgPackageJSON(libName);
        var entryFile = lib.root + "/" + libPKG.lib.entryFile;
        var ex = path.extname(entryFile);
        return entryFile.slice(0, entryFile.lastIndexOf(ex));
    };
    return LibraryController;
}());
exports.LibraryController = LibraryController;
//# sourceMappingURL=library-controller.js.map